/*
 Copyright (c) 2006 - 2007 Eric J. Feminella  <eric@ericfeminella.com>
 All rights reserved.
  
 Permission is hereby granted, free of charge, to any person obtaining a copy 
 of this software and associated documentation files (the "Software"), to deal 
 in the Software without restriction, including without limitation the rights 
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is furnished 
 to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included in all 
 copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
 PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
 OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

 @internal
 */

package com.ericfeminella.utils.iterators 
{
	import mx.collections.CursorBookmark;
	import mx.collections.ICollectionView;
	import mx.collections.IViewCursor;

    /**
     * Concrete Iterator implementation which provides an API for 
     * traversing an ICollectionView
     * 
     * @see mx.collections.ICollectionView
     * @see mx.collections.IList
     * @see mx.collections.CursorBookmark
     * @see mx.collections.IViewCursor
     */
	public class CollectionIterator implements IIterator
	{
		/**
		 * Defines the ICollectionView aggregate object instance
		 */
		protected var collection:ICollectionView;
		
		/**
		 * Defines the IViewCursor instance to the aggregate
		 */
		protected var cursor:IViewCursor;
		
		/**
		 * Contains the current indexed position in the aggregate
		 */
		private var index:int;
		
		/** 
		 * Instantiates a new CollectionIterator instance
		 * 
         * @example A simple CollectionIterator example is as follows:
         * <listing version="3.0" >
         * 
	     * var collection:ICollectionView = new ArrayCollection(["A","B","C"]);
	     * var it:IIterator = new CollectionIterator( collection );
	     * 
	     * while ( it.hasNext() )
	     * {
	     *     trace( it.next(), it.position());
	     * }
	     * 
	     * //outputs:
	     * //A, 0
	     * //B, 1
	     * //C, 2 
         * </listing>
	     * 
		 * @param the collection in which to iterate over
		 */
		public function CollectionIterator(collection:ICollectionView = null)
		{
			this.setAggregate( collection );			
		}
        
		/**
		 * Sets the collection in which to iterate over
		 * 
		 * @param an ICollectionView instance
		 * @inheritDoc
		 */
        public function setAggregate(aggregate:Object = null) : void
        {
        	if (aggregate != null)
        	{
	        	this.collection = aggregate as ICollectionView;
	        	this.cursor = this.collection.createCursor();      		
        	}
        }
        
		/**
		 * Determines if there are elements remaining in the ICollectionView
		 * 
		 * @inheritDoc
		 */	
		public function hasNext() : Boolean
		{
			if (this.cursor.beforeFirst || this.cursor.afterLast)
			{
				return false;
			}
			return true;
		}
		
		/**
		 * Returns the next item in the ICollectionView
		 * 
		 * @inheritDoc
		 */
		public function next() : *
		{
			var current:Object = this.cursor.current;
			
			this.index = this.cursor.bookmark.getViewIndex();
			this.cursor.moveNext();
			
			return current;
		}
		
		/**
		 * Resets the IViewCursor to a zero based indexed position
		 * 
		 * @see mx.collections.CursorBookmark#FIRST
		 */ 
	    public function reset() : void
	    {
	    	this.cursor.seek( CursorBookmark.FIRST );
	    }
	    
		/**
		 * Determines the IViewCursor position of the ICollectionView
		 * 
		 * @inheritDoc
		 */
	    public function position() : int
		{
			return index;
		}
	}
}






