/*
 Copyright (c) 2006 - 2007 Eric J. Feminella  <eric@ericfeminella.com>
 All rights reserved.
  
 Permission is hereby granted, free of charge, to any person obtaining a copy 
 of this software and associated documentation files (the "Software"), to deal 
 in the Software without restriction, including without limitation the rights 
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is furnished 
 to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included in all 
 copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
 PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
 OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

 @internal
 */
 
package com.ericfeminella.utils.iterators 
{
    /** 
     * Iterator specific Factory Pattern implementation which defines an 
     * API for handling specific Iterator instantiation
     * 
     * @see com.ericfeminella.utils.iterators.IteratorTypes
     */
	public final class IteratorFactory 
	{
        /**
         * Static method which handles Iterator instantiation based on static 
         * constants defined in IteratorTypes:
         * 
		 * <p>
		 * As a best practice, use a constant defined by IteratorTypes:
		 * 
		 * <ul>
		 *    <li>IteratorTypes.OBJECT_ITERATOR;</li>
		 *    <li>IteratorTypes.ARRAY_ITERATOR;</li>
		 *    <li>IteratorTypes.COLLECTION_ITERATOR;</li>
		 * </ul>
		 * </p>
		 * 
	     * @example For instance, a client would typically create an iterator as follows
	     * 
	     * <listing version="3.0" >
	     * 
		 * import com.ericfeminella.utils.iterators.IIterator;
		 * import com.ericfeminella.utils.iterators.IteratorFactory;
		 * import com.ericfeminella.utils.iterators.IteratorTypes;
		 * import mx.collections.ArrayCollection;
		 * 
	     * var collection:ArrayCollection = new ArrayCollection(["A","B","C"]);
	     * var it:IIterator = IteratorFactory.getIterator( IteratorTypes.COLLECTION_ITERATOR );
	     * it.setAggregate( collection );
	     * 
		 * while ( iterator.hasNext() ) 
		 * {
	     *     trace( it.next(), it.position());
		 * }
		 * 
		 * //A, 0
		 * //B, 1
		 * //C, 2
		 * 
		 * iterator = IteratorFactory.getIterator(IteratorTypes.ARRAY_ITERATOR);
		 * iterator.setAggregate(["A", "B", "C"]);
		 * 
		 * while (iterator.hasNext())
		 * {
	     *    trace(iterator.next());
		 * }
		 * 
		 * //A, 0
		 * //B, 1
		 * //C, 2
		 * 
		 * </listing>
		 * 
		 * <p>
		 * By typing an Iterator instance as an IIterator, clients have the flexibility
		 * of changing the concrete iterator at runtime
		 * </p>
		 * 
         * @param   a static constant representing a valid IteratorType
         * @return  concrete iterator implementation as specified by type argument
         * @see     <code>com.ericfeminella.utils.iterators#IteratorTypes</code>
         * 
         */
		public static function getIterator(type:String) : IIterator
		{
			switch( type )
			{
				case IteratorTypes.OBJECT_ITERATOR:
				  return new ObjectIterator();
				  break;
				case IteratorTypes.ARRAY_ITERATOR:
				  return new ArrayIterator();
				  break;
				case IteratorTypes.COLLECTION_ITERATOR:
				  return new CollectionIterator();
				  break;
			}
			return null;
		}
	}
}

